@@ -9,7 +9,7 @@
 #ifndef BROTLI_ENC_CLUSTER_H_
 #define BROTLI_ENC_CLUSTER_H_
 
-#include <brotli/types.h>
+#include "../common/types.h"
 #include "./histogram.h"
 #include "./memory.h"
 #include "./port.h"
