@@ -10,7 +10,7 @@
 #ifndef BROTLI_ENC_LITERAL_COST_H_
 #define BROTLI_ENC_LITERAL_COST_H_
 
-#include <brotli/types.h>
+#include "../common/types.h"
 #include "./port.h"
 
 #if defined(__cplusplus) || defined(c_plusplus)
@@ -18,7 +18,7 @@ extern "C" {
 #endif
 
 /* Estimates how many bits the literals in the interval [pos, pos + len) in the
-   ring-buffer (data, mask) will take entropy coded and writes these estimates
+   ringbuffer (data, mask) will take entropy coded and writes these estimates
    to the cost[0..len) array. */
 BROTLI_INTERNAL void BrotliEstimateBitCostsForLiterals(
     size_t pos, size_t len, size_t mask, const uint8_t *data, float *cost);
