@@ -10,7 +10,7 @@
 #include "./metablock.h"
 
 #include "../common/constants.h"
-#include <brotli/types.h>
+#include "../common/types.h"
 #include "./bit_cost.h"
 #include "./block_splitter.h"
 #include "./cluster.h"
@@ -40,11 +40,10 @@ void BrotliBuildMetaBlock(MemoryManager* m,
   static const size_t kMaxNumberOfHistograms = 256;
   HistogramDistance* distance_histograms;
   HistogramLiteral* literal_histograms;
-  ContextType* literal_context_modes = NULL;
-  size_t literal_histograms_size;
-  size_t distance_histograms_size;
+  ContextType* literal_context_modes;
+  size_t num_literal_contexts;
+  size_t num_distance_contexts;
   size_t i;
-  size_t literal_context_multiplier = 1;
 
   BrotliSplitBlock(m, cmds, num_commands,
                    ringbuffer, pos, mask, params,
@@ -53,29 +52,20 @@ void BrotliBuildMetaBlock(MemoryManager* m,
                    &mb->distance_split);
   if (BROTLI_IS_OOM(m)) return;
 
-  if (!params->disable_literal_context_modeling) {
-    literal_context_multiplier = 1 << BROTLI_LITERAL_CONTEXT_BITS;
-    literal_context_modes =
-        BROTLI_ALLOC(m, ContextType, mb->literal_split.num_types);
-    if (BROTLI_IS_OOM(m)) return;
-    for (i = 0; i < mb->literal_split.num_types; ++i) {
-      literal_context_modes[i] = literal_context_mode;
-    }
+  literal_context_modes =
+      BROTLI_ALLOC(m, ContextType, mb->literal_split.num_types);
+  if (BROTLI_IS_OOM(m)) return;
+  for (i = 0; i < mb->literal_split.num_types; ++i) {
+    literal_context_modes[i] = literal_context_mode;
   }
 
-  literal_histograms_size =
-      mb->literal_split.num_types * literal_context_multiplier;
-  literal_histograms =
-      BROTLI_ALLOC(m, HistogramLiteral, literal_histograms_size);
-  if (BROTLI_IS_OOM(m)) return;
-  ClearHistogramsLiteral(literal_histograms, literal_histograms_size);
-
-  distance_histograms_size =
+  num_literal_contexts =
+      mb->literal_split.num_types << BROTLI_LITERAL_CONTEXT_BITS;
+  num_distance_contexts =
       mb->distance_split.num_types << BROTLI_DISTANCE_CONTEXT_BITS;
-  distance_histograms =
-      BROTLI_ALLOC(m, HistogramDistance, distance_histograms_size);
+  literal_histograms = BROTLI_ALLOC(m, HistogramLiteral, num_literal_contexts);
   if (BROTLI_IS_OOM(m)) return;
-  ClearHistogramsDistance(distance_histograms, distance_histograms_size);
+  ClearHistogramsLiteral(literal_histograms, num_literal_contexts);
 
   assert(mb->command_histograms == 0);
   mb->command_histograms_size = mb->command_split.num_types;
@@ -83,7 +73,10 @@ void BrotliBuildMetaBlock(MemoryManager* m,
       BROTLI_ALLOC(m, HistogramCommand, mb->command_histograms_size);
   if (BROTLI_IS_OOM(m)) return;
   ClearHistogramsCommand(mb->command_histograms, mb->command_histograms_size);
-
+  distance_histograms =
+      BROTLI_ALLOC(m, HistogramDistance, num_distance_contexts);
+  if (BROTLI_IS_OOM(m)) return;
+  ClearHistogramsDistance(distance_histograms, num_distance_contexts);
   BrotliBuildHistogramsWithContext(cmds, num_commands,
       &mb->literal_split, &mb->command_split, &mb->distance_split,
       ringbuffer, pos, mask, prev_byte, prev_byte2, literal_context_modes,
@@ -96,44 +89,31 @@ void BrotliBuildMetaBlock(MemoryManager* m,
   mb->literal_context_map =
       BROTLI_ALLOC(m, uint32_t, mb->literal_context_map_size);
   if (BROTLI_IS_OOM(m)) return;
-
   assert(mb->literal_histograms == 0);
   mb->literal_histograms_size = mb->literal_context_map_size;
   mb->literal_histograms =
       BROTLI_ALLOC(m, HistogramLiteral, mb->literal_histograms_size);
   if (BROTLI_IS_OOM(m)) return;
-
-  BrotliClusterHistogramsLiteral(m, literal_histograms, literal_histograms_size,
-      kMaxNumberOfHistograms, mb->literal_histograms,
-      &mb->literal_histograms_size, mb->literal_context_map);
+  BrotliClusterHistogramsLiteral(m, literal_histograms,
+                                 mb->literal_context_map_size,
+                                 kMaxNumberOfHistograms,
+                                 mb->literal_histograms,
+                                 &mb->literal_histograms_size,
+                                 mb->literal_context_map);
   if (BROTLI_IS_OOM(m)) return;
   BROTLI_FREE(m, literal_histograms);
 
-  if (params->disable_literal_context_modeling) {
-    /* Distribute assignment to all contexts. */
-    for (i = mb->literal_split.num_types; i != 0;) {
-      size_t j = 0;
-      i--;
-      for (; j < (1 << BROTLI_LITERAL_CONTEXT_BITS); j++) {
-        mb->literal_context_map[(i << BROTLI_LITERAL_CONTEXT_BITS) + j] =
-            mb->literal_context_map[i];
-      }
-    }
-  }
-
   assert(mb->distance_context_map == 0);
   mb->distance_context_map_size =
       mb->distance_split.num_types << BROTLI_DISTANCE_CONTEXT_BITS;
   mb->distance_context_map =
       BROTLI_ALLOC(m, uint32_t, mb->distance_context_map_size);
   if (BROTLI_IS_OOM(m)) return;
-
   assert(mb->distance_histograms == 0);
   mb->distance_histograms_size = mb->distance_context_map_size;
   mb->distance_histograms =
       BROTLI_ALLOC(m, HistogramDistance, mb->distance_histograms_size);
   if (BROTLI_IS_OOM(m)) return;
-
   BrotliClusterHistogramsDistance(m, distance_histograms,
                                   mb->distance_context_map_size,
                                   kMaxNumberOfHistograms,
@@ -156,7 +136,53 @@ void BrotliBuildMetaBlock(MemoryManager* m,
 #include "./metablock_inc.h"  /* NOLINT(build/include) */
 #undef FN
 
-#define BROTLI_MAX_STATIC_CONTEXTS 13
+void BrotliBuildMetaBlockGreedy(MemoryManager* m,
+                                const uint8_t* ringbuffer,
+                                size_t pos,
+                                size_t mask,
+                                const Command *commands,
+                                size_t n_commands,
+                                MetaBlockSplit* mb) {
+  BlockSplitterLiteral lit_blocks;
+  BlockSplitterCommand cmd_blocks;
+  BlockSplitterDistance dist_blocks;
+  size_t num_literals = 0;
+  size_t i;
+  for (i = 0; i < n_commands; ++i) {
+    num_literals += commands[i].insert_len_;
+  }
+
+  InitBlockSplitterLiteral(m, &lit_blocks, 256, 512, 400.0, num_literals,
+      &mb->literal_split, &mb->literal_histograms,
+      &mb->literal_histograms_size);
+  if (BROTLI_IS_OOM(m)) return;
+  InitBlockSplitterCommand(m, &cmd_blocks, BROTLI_NUM_COMMAND_SYMBOLS, 1024,
+      500.0, n_commands, &mb->command_split, &mb->command_histograms,
+      &mb->command_histograms_size);
+  if (BROTLI_IS_OOM(m)) return;
+  InitBlockSplitterDistance(m, &dist_blocks, 64, 512, 100.0, n_commands,
+      &mb->distance_split, &mb->distance_histograms,
+      &mb->distance_histograms_size);
+  if (BROTLI_IS_OOM(m)) return;
+
+  for (i = 0; i < n_commands; ++i) {
+    const Command cmd = commands[i];
+    size_t j;
+    BlockSplitterAddSymbolCommand(&cmd_blocks, cmd.cmd_prefix_);
+    for (j = cmd.insert_len_; j != 0; --j) {
+      BlockSplitterAddSymbolLiteral(&lit_blocks, ringbuffer[pos & mask]);
+      ++pos;
+    }
+    pos += CommandCopyLen(&cmd);
+    if (CommandCopyLen(&cmd) && cmd.cmd_prefix_ >= 128) {
+      BlockSplitterAddSymbolDistance(&dist_blocks, cmd.dist_prefix_);
+    }
+  }
+
+  BlockSplitterFinishBlockLiteral(&lit_blocks, /* is_final = */ BROTLI_TRUE);
+  BlockSplitterFinishBlockCommand(&cmd_blocks, /* is_final = */ BROTLI_TRUE);
+  BlockSplitterFinishBlockDistance(&dist_blocks, /* is_final = */ BROTLI_TRUE);
+}
 
 /* Greedy block splitter for one block category (literal, command or distance).
    Gathers histograms for all context buckets. */
@@ -188,7 +214,7 @@ typedef struct ContextBlockSplitter {
   /* Offset of the histograms of the previous two block types. */
   size_t last_histogram_ix_[2];
   /* Entropy of the previous two block types. */
-  double last_entropy_[2 * BROTLI_MAX_STATIC_CONTEXTS];
+  double* last_entropy_;
   /* The number of times we merged the current block with the last one. */
   size_t merge_last_count_;
 } ContextBlockSplitter;
@@ -200,7 +226,6 @@ static void InitContextBlockSplitter(
     size_t* histograms_size) {
   size_t max_num_blocks = num_symbols / min_block_size + 1;
   size_t max_num_types;
-  assert(num_contexts <= BROTLI_MAX_STATIC_CONTEXTS);
 
   self->alphabet_size_ = alphabet_size;
   self->num_contexts_ = num_contexts;
@@ -225,23 +250,29 @@ static void InitContextBlockSplitter(
       split->lengths, split->lengths_alloc_size, max_num_blocks);
   if (BROTLI_IS_OOM(m)) return;
   split->num_blocks = max_num_blocks;
+  self->last_entropy_ = BROTLI_ALLOC(m, double, 2 * num_contexts);
   if (BROTLI_IS_OOM(m)) return;
   assert(*histograms == 0);
   *histograms_size = max_num_types * num_contexts;
   *histograms = BROTLI_ALLOC(m, HistogramLiteral, *histograms_size);
   self->histograms_ = *histograms;
   if (BROTLI_IS_OOM(m)) return;
-  /* Clear only current histogram. */
+  /* Clear only current historgram. */
   ClearHistogramsLiteral(&self->histograms_[0], num_contexts);
   self->last_histogram_ix_[0] = self->last_histogram_ix_[1] = 0;
 }
 
+static void CleanupContextBlockSplitter(
+    MemoryManager* m, ContextBlockSplitter* self) {
+  BROTLI_FREE(m, self->last_entropy_);
+}
+
 /* Does either of three things:
      (1) emits the current block with a new block type;
      (2) emits the current block with the type of the second last block;
      (3) merges the current block with the last block. */
 static void ContextBlockSplitterFinishBlock(
-    ContextBlockSplitter* self, MemoryManager* m, BROTLI_BOOL is_final) {
+    MemoryManager* m, ContextBlockSplitter* self, BROTLI_BOOL is_final) {
   BlockSplit* split = self->split_;
   const size_t num_contexts = self->num_contexts_;
   double* last_entropy = self->last_entropy_;
@@ -274,10 +305,10 @@ static void ContextBlockSplitterFinishBlock(
        respective set of histograms for the last and second last block types.
        Decide over the split based on the total reduction of entropy across
        all contexts. */
-    double entropy[BROTLI_MAX_STATIC_CONTEXTS];
+    double* entropy = BROTLI_ALLOC(m, double, num_contexts);
     HistogramLiteral* combined_histo =
         BROTLI_ALLOC(m, HistogramLiteral, 2 * num_contexts);
-    double combined_entropy[2 * BROTLI_MAX_STATIC_CONTEXTS];
+    double* combined_entropy = BROTLI_ALLOC(m, double, 2 * num_contexts);
     double diff[2] = { 0.0 };
     size_t i;
     if (BROTLI_IS_OOM(m)) return;
@@ -352,7 +383,9 @@ static void ContextBlockSplitterFinishBlock(
         self->target_block_size_ += self->min_block_size_;
       }
     }
+    BROTLI_FREE(m, combined_entropy);
     BROTLI_FREE(m, combined_histo);
+    BROTLI_FREE(m, entropy);
   }
   if (is_final) {
     *self->histograms_size_ = split->num_types * num_contexts;
@@ -362,49 +395,30 @@ static void ContextBlockSplitterFinishBlock(
 
 /* Adds the next symbol to the current block type and context. When the
    current block reaches the target size, decides on merging the block. */
-static void ContextBlockSplitterAddSymbol(
-    ContextBlockSplitter* self, MemoryManager* m,
-    size_t symbol, size_t context) {
+static void ContextBlockSplitterAddSymbol(MemoryManager* m,
+    ContextBlockSplitter* self, size_t symbol, size_t context) {
   HistogramAddLiteral(&self->histograms_[self->curr_histogram_ix_ + context],
       symbol);
   ++self->block_size_;
   if (self->block_size_ == self->target_block_size_) {
-    ContextBlockSplitterFinishBlock(self, m, /* is_final = */ BROTLI_FALSE);
+    ContextBlockSplitterFinishBlock(m, self, /* is_final = */ BROTLI_FALSE);
     if (BROTLI_IS_OOM(m)) return;
   }
 }
 
-static void MapStaticContexts(MemoryManager* m,
-                              size_t num_contexts,
-                              const uint32_t* static_context_map,
-                              MetaBlockSplit* mb) {
-  size_t i;
-  assert(mb->literal_context_map == 0);
-  mb->literal_context_map_size =
-      mb->literal_split.num_types << BROTLI_LITERAL_CONTEXT_BITS;
-  mb->literal_context_map =
-      BROTLI_ALLOC(m, uint32_t, mb->literal_context_map_size);
-  if (BROTLI_IS_OOM(m)) return;
-
-  for (i = 0; i < mb->literal_split.num_types; ++i) {
-    uint32_t offset = (uint32_t)(i * num_contexts);
-    size_t j;
-    for (j = 0; j < (1u << BROTLI_LITERAL_CONTEXT_BITS); ++j) {
-      mb->literal_context_map[(i << BROTLI_LITERAL_CONTEXT_BITS) + j] =
-          offset + static_context_map[j];
-    }
-  }
-}
-
-static BROTLI_INLINE void BrotliBuildMetaBlockGreedyInternal(
-    MemoryManager* m, const uint8_t* ringbuffer, size_t pos, size_t mask,
-    uint8_t prev_byte, uint8_t prev_byte2, ContextType literal_context_mode,
-    const size_t num_contexts, const uint32_t* static_context_map,
-    const Command *commands, size_t n_commands, MetaBlockSplit* mb) {
-  union {
-    BlockSplitterLiteral plain;
-    ContextBlockSplitter ctx;
-  } lit_blocks;
+void BrotliBuildMetaBlockGreedyWithContexts(MemoryManager* m,
+                                            const uint8_t* ringbuffer,
+                                            size_t pos,
+                                            size_t mask,
+                                            uint8_t prev_byte,
+                                            uint8_t prev_byte2,
+                                            ContextType literal_context_mode,
+                                            size_t num_contexts,
+                                            const uint32_t* static_context_map,
+                                            const Command *commands,
+                                            size_t n_commands,
+                                            MetaBlockSplit* mb) {
+  ContextBlockSplitter lit_blocks;
   BlockSplitterCommand cmd_blocks;
   BlockSplitterDistance dist_blocks;
   size_t num_literals = 0;
@@ -413,15 +427,9 @@ static BROTLI_INLINE void BrotliBuildMetaBlockGreedyInternal(
     num_literals += commands[i].insert_len_;
   }
 
-  if (num_contexts == 1) {
-    InitBlockSplitterLiteral(m, &lit_blocks.plain, 256, 512, 400.0,
-        num_literals, &mb->literal_split, &mb->literal_histograms,
-        &mb->literal_histograms_size);
-  } else {
-    InitContextBlockSplitter(m, &lit_blocks.ctx, 256, num_contexts, 512, 400.0,
-        num_literals, &mb->literal_split, &mb->literal_histograms,
-        &mb->literal_histograms_size);
-  }
+  InitContextBlockSplitter(m, &lit_blocks, 256, num_contexts, 512, 400.0,
+      num_literals, &mb->literal_split, &mb->literal_histograms,
+      &mb->literal_histograms_size);
   if (BROTLI_IS_OOM(m)) return;
   InitBlockSplitterCommand(m, &cmd_blocks, BROTLI_NUM_COMMAND_SYMBOLS, 1024,
       500.0, n_commands, &mb->command_split, &mb->command_histograms,
@@ -437,16 +445,12 @@ static BROTLI_INLINE void BrotliBuildMetaBlockGreedyInternal(
     size_t j;
     BlockSplitterAddSymbolCommand(&cmd_blocks, cmd.cmd_prefix_);
     for (j = cmd.insert_len_; j != 0; --j) {
+      size_t context = Context(prev_byte, prev_byte2, literal_context_mode);
       uint8_t literal = ringbuffer[pos & mask];
-      if (num_contexts == 1) {
-        BlockSplitterAddSymbolLiteral(&lit_blocks.plain, literal);
-      } else {
-        size_t context = Context(prev_byte, prev_byte2, literal_context_mode);
-        ContextBlockSplitterAddSymbol(&lit_blocks.ctx, m, literal,
-                                      static_context_map[context]);
-        if (BROTLI_IS_OOM(m)) return;
-      }
+      ContextBlockSplitterAddSymbol(
+          m, &lit_blocks, literal, static_context_map[context]);
       prev_byte2 = prev_byte;
+      if (BROTLI_IS_OOM(m)) return;
       prev_byte = literal;
       ++pos;
     }
@@ -460,41 +464,25 @@ static BROTLI_INLINE void BrotliBuildMetaBlockGreedyInternal(
     }
   }
 
-  if (num_contexts == 1) {
-    BlockSplitterFinishBlockLiteral(
-        &lit_blocks.plain, /* is_final = */ BROTLI_TRUE);
-  } else {
-    ContextBlockSplitterFinishBlock(
-        &lit_blocks.ctx, m, /* is_final = */ BROTLI_TRUE);
-    if (BROTLI_IS_OOM(m)) return;
-  }
+  ContextBlockSplitterFinishBlock(m, &lit_blocks, /* is_final = */ BROTLI_TRUE);
+  if (BROTLI_IS_OOM(m)) return;
+  CleanupContextBlockSplitter(m, &lit_blocks);
   BlockSplitterFinishBlockCommand(&cmd_blocks, /* is_final = */ BROTLI_TRUE);
   BlockSplitterFinishBlockDistance(&dist_blocks, /* is_final = */ BROTLI_TRUE);
 
-  if (num_contexts > 1) {
-    MapStaticContexts(m, num_contexts, static_context_map, mb);
-  }
-}
+  assert(mb->literal_context_map == 0);
+  mb->literal_context_map_size =
+      mb->literal_split.num_types << BROTLI_LITERAL_CONTEXT_BITS;
+  mb->literal_context_map =
+      BROTLI_ALLOC(m, uint32_t, mb->literal_context_map_size);
+  if (BROTLI_IS_OOM(m)) return;
 
-void BrotliBuildMetaBlockGreedy(MemoryManager* m,
-                                const uint8_t* ringbuffer,
-                                size_t pos,
-                                size_t mask,
-                                uint8_t prev_byte,
-                                uint8_t prev_byte2,
-                                ContextType literal_context_mode,
-                                size_t num_contexts,
-                                const uint32_t* static_context_map,
-                                const Command* commands,
-                                size_t n_commands,
-                                MetaBlockSplit* mb) {
-  if (num_contexts == 1) {
-    BrotliBuildMetaBlockGreedyInternal(m, ringbuffer, pos, mask, prev_byte,
-        prev_byte2, literal_context_mode, 1, NULL, commands, n_commands, mb);
-  } else {
-    BrotliBuildMetaBlockGreedyInternal(m, ringbuffer, pos, mask, prev_byte,
-        prev_byte2, literal_context_mode, num_contexts, static_context_map,
-        commands, n_commands, mb);
+  for (i = 0; i < mb->literal_split.num_types; ++i) {
+    size_t j;
+    for (j = 0; j < (1u << BROTLI_LITERAL_CONTEXT_BITS); ++j) {
+      mb->literal_context_map[(i << BROTLI_LITERAL_CONTEXT_BITS) + j] =
+          (uint32_t)(i * num_contexts) + static_context_map[j];
+    }
   }
 }
 
@@ -514,8 +502,7 @@ void BrotliOptimizeHistograms(size_t num_direct_distance_codes,
                                       good_for_rle);
   }
   num_distance_codes = BROTLI_NUM_DISTANCE_SHORT_CODES +
-      num_direct_distance_codes +
-      ((2 * BROTLI_MAX_DISTANCE_BITS) << distance_postfix_bits);
+      num_direct_distance_codes + (48u << distance_postfix_bits);
   for (i = 0; i < mb->distance_histograms_size; ++i) {
     BrotliOptimizeHuffmanCountsForRle(num_distance_codes,
                                       mb->distance_histograms[i].data_,
