@@ -9,8 +9,7 @@
 #ifndef BROTLI_ENC_STATIC_DICT_H_
 #define BROTLI_ENC_STATIC_DICT_H_
 
-#include "../common/dictionary.h"
-#include <brotli/types.h>
+#include "../common/types.h"
 #include "./port.h"
 
 #if defined(__cplusplus) || defined(c_plusplus)
@@ -28,7 +27,6 @@ static const uint32_t kInvalidMatch = 0xfffffff;
      matches array is at least BROTLI_MAX_STATIC_DICTIONARY_MATCH_LEN + 1 long
      all elements are initialized to kInvalidMatch */
 BROTLI_INTERNAL BROTLI_BOOL BrotliFindAllStaticDictionaryMatches(
-    const BrotliDictionary* dictionary,
     const uint8_t* data, size_t min_length, size_t max_length,
     uint32_t* matches);
 
