@@ -1,11 +1,11 @@
-#!/usr/bin/env bash
+#!/bin/bash
 #
 # Test that the brotli command-line tool can decompress old brotli-compressed
 # files.
 
 set -o errexit
 
-BROTLI=bin/brotli
+BRO=bin/bro
 TMP_DIR=bin/tmp
 
 for file in tests/testdata/*.compressed*; do
@@ -13,10 +13,10 @@ for file in tests/testdata/*.compressed*; do
   expected=${file%.compressed*}
   uncompressed=${TMP_DIR}/${expected##*/}.uncompressed
   echo $uncompressed
-  $BROTLI $file -fdo $uncompressed
+  $BRO -f -d -i $file -o $uncompressed
   diff -q $uncompressed $expected
   # Test the streaming version
-  cat $file | $BROTLI -dc > $uncompressed
+  cat $file | $BRO -d > $uncompressed
   diff -q $uncompressed $expected
   rm -f $uncompressed
 done
